@@ -338,9 +338,12 @@ public:
         double lp = score_segmentation( beam[0].segmentation, out_sequence );
 
         // fill other (dummy) output parameters
-        component_rects->push_back(Rect(0,0,src.cols,src.rows));
-        component_texts->push_back(out_sequence);
-        component_confidences->push_back((float)exp(lp));
+        if (component_rects != NULL)
+            component_rects->push_back(Rect(0,0,src.cols,src.rows));
+        if (component_texts != NULL)
+            component_texts->push_back(out_sequence);
+        if (component_confidences != NULL)
+            component_confidences->push_back((float)exp(lp));
 
         return;
     }
@@ -611,7 +614,6 @@ void OCRBeamSearchClassifierCNN::eval( InputArray _src, vector< vector<double> >
 
         img = src(Rect(Point(x_c,0),Size(window_size,window_size)));
 
-        int patch_count = 0;
         vector< vector<double> > data_pool(9);
 
 
@@ -652,7 +654,6 @@ void OCRBeamSearchClassifierCNN::eval( InputArray _src, vector< vector<double> >
                             data_pool[7].insert(data_pool[7].end(),patch.begin(),patch.end());
                         if ((quad_id == 19)||(quad_id == 24)||(quad_id == 20)||(quad_id == 25))
                             data_pool[8].insert(data_pool[8].end(),patch.begin(),patch.end());
-                        patch_count++;
                     }
                 }
 
